#include "xmedia.h"
#include "ui_xmedia.h"
#include <iostream>
#include <QtGui/QApplication>


xMedia::xMedia(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::xMediaClass)
{
    ui->setupUi(this);

    this->system = new xmedia_system();
    system->start(QThread::LowestPriority);


    ItemPosition = 0;
    new QListWidgetItem(QIcon(":/images/extras.png"), "Extras", ui->listWidget, 1002);
    new QListWidgetItem(QIcon(":/images/home.png"), "Home", ui->listWidget, 2001);
    new QListWidgetItem(QIcon(":/images/music.png"), "Multimedia", ui->listWidget, 2002);
    new QListWidgetItem(QIcon(":/images/video.png"), "Streams", ui->listWidget, 2003);
    new QListWidgetItem(QIcon(":/images/extras.png"), "Extras", ui->listWidget, 2005);
    new QListWidgetItem(QIcon(":/images/home.png"), "Home", ui->listWidget, 1002);

    QMovie *loading_gif = new QMovie(":/images/loader.gif");
    this->ui->loading->setMovie(loading_gif);
    loading_gif->setSpeed(100);
    loading_gif->start();

    this->scrollToItem(1);

    connect(ui->listWidget, SIGNAL(currentItemChanged (QListWidgetItem*, QListWidgetItem*)), this, SLOT(itemHighlighted(QListWidgetItem*, QListWidgetItem*)));
    connect(ui->listWidget, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(itemSelected(QListWidgetItem*)));
    connect(ui->listWidget, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(itemSelected(QListWidgetItem*)));

    connect(&UserInterface, SIGNAL(commandInfoReceived(unsigned char, unsigned char,unsigned char *)),
            this, SLOT(commandInfoReceived(unsigned char,unsigned char , unsigned char *)));
    connect(&UserInterface, SIGNAL(commandPowerManagmentReceived(unsigned char, unsigned char, unsigned char *)),
        this, SLOT(commandPowerManagmentReceived(unsigned char,unsigned char , unsigned char *)));
    connect(&UserInterface, SIGNAL(commandMouseEmulationReceived(unsigned char,unsigned char , unsigned char *)),
        this, SLOT(commandMouseEmulationReceived(unsigned char,unsigned char , unsigned char *)));
    connect(&UserInterface, SIGNAL(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)),
        this, SLOT(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)));
}

xMedia::~xMedia()
{
    delete ui;
}

void xMedia::itemHighlighted(QListWidgetItem *mitem, QListWidgetItem* prev)
{
    if(mitem)
    {
        if(mitem->type() == 1002 && prev != 0)
        {
            ui->listWidget->scrollToItem(prev, QAbstractItemView::PositionAtCenter);
            prev->setSelected(true);
        }
        else
        {
            ui->listWidget->scrollToItem(mitem, QAbstractItemView::PositionAtCenter);
        }
    }
}

void xMedia::itemSelected(QListWidgetItem *mitem)
{
    unsigned int item = mitem->type() - 2000;

    if(item == 1)
    {
        this->ui->stackedWidget->setCurrentIndex(1);
        std::cout<<"xmedia: Opening homescreen!"<<std::endl;

        xmedia_home *home = new xmedia_home(this);

        disconnect(&UserInterface, SIGNAL(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)),
        this, SLOT(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)));
        connect(&UserInterface, SIGNAL(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)),
            home, SLOT(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)));
        connect(this->system, SIGNAL(timeUpdated(QTime, QDate)),
            home, SLOT(timeUpdated(QTime, QDate)));

        home->showFullScreen();
        home->exec();

        connect(&UserInterface, SIGNAL(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)),
            this, SLOT(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)));

        this->ui->stackedWidget->setCurrentIndex(0);

        delete home;

    }
    else if(item == 2)
    {
        this->ui->stackedWidget->setCurrentIndex(1);
        std::cout<<"xmedia: Playing music!"<<std::endl;

        qApp->processEvents(QEventLoop::AllEvents);

        xmedia_music *music = new xmedia_music(this);

        qApp->processEvents(QEventLoop::AllEvents);

        disconnect(&UserInterface, SIGNAL(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)),
        this, SLOT(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)));
        connect(&UserInterface, SIGNAL(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)),
            music, SLOT(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)));

        qApp->processEvents(QEventLoop::AllEvents);

        music->showFullScreen();

        qApp->processEvents(QEventLoop::AllEvents);

        music->exec();
        connect(&UserInterface, SIGNAL(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)),
            this, SLOT(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)));

        qApp->processEvents(QEventLoop::AllEvents);

        this->ui->stackedWidget->setCurrentIndex(0);

        delete music;
    }
    else if(item == 3)
    {
        this->ui->stackedWidget->setCurrentIndex(1);
        std::cout<<"xmedia: Playing streams!"<<std::endl;

        qApp->processEvents(QEventLoop::AllEvents);

        xmedia_stream *stream = new xmedia_stream(this);

        qApp->processEvents(QEventLoop::AllEvents);

        disconnect(&UserInterface, SIGNAL(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)),
        this, SLOT(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)));
        connect(&UserInterface, SIGNAL(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)),
            stream, SLOT(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)));

        qApp->processEvents(QEventLoop::AllEvents);

        stream->showFullScreen();

        qApp->processEvents(QEventLoop::AllEvents);

        stream->exec();
        connect(&UserInterface, SIGNAL(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)),
            this, SLOT(commandUserInterfaceReceived(unsigned char,unsigned char , unsigned char *)));

        qApp->processEvents(QEventLoop::AllEvents);

        this->ui->stackedWidget->setCurrentIndex(0);

        delete stream;

    }
}


void xMedia::commandInfoReceived(unsigned char command, unsigned char length, unsigned char *data )
{
    if(command == 0x11)
    {
        UserInterface.sendCommand(0x01, 0, 0);
    }
    std::cout<<"xmedia: Info Command "<<(int)command<<" received!"<<std::endl;
}
void xMedia::commandPowerManagmentReceived(unsigned char command, unsigned char length, unsigned char *data )
{
    std::cout<<"xmedia: PM Command "<<(int)command<<" received!"<<std::endl;
}

int xMedia::scrollToItem(int relative)
{
    int new_row = relative;

    if(ui->listWidget->currentItem())
    {
        new_row += ui->listWidget->row(ui->listWidget->currentItem());
    }

    if(new_row == 0)
    {
        new_row = 4;
    }
    if(new_row >= 5)
    {
        new_row = 1;
    }

    if(new_row >= 0 && ui->listWidget->item(new_row) && ui->listWidget->item(new_row)->type() != 1002)
    {
        ui->listWidget->setCurrentItem(ui->listWidget->item(new_row));
    }

    return new_row;
}

void xMedia::commandUserInterfaceReceived(unsigned char command, unsigned char length, unsigned char *data )
{
    if(command == XUI_LEFT || command == XUI_4WAY_LEFT)
    {
        scrollToItem(-1);
    }
    else if(command == XUI_RIGHT || command == XUI_4WAY_RIGHT)
    {
        scrollToItem(1);
    }
    else if(command == XUI_OK || command == XUI_4WAY_UP || command == XUI_4WAY_DOWN)
    {
        itemSelected(ui->listWidget->currentItem());
    }

    //std::cout<<"xmedia: UI Command "<<(int)command<<" received!"<<std::endl;
}
void xMedia::commandMouseEmulationReceived(unsigned char command, unsigned char length, unsigned char *data )
{
    std::cout<<"xmedia: ME Command "<<(int)command<<" received!"<<std::endl;
}
void xMedia::transmissionError(xsieError error)
{
    std::cout<<"xmedia: Packet Error:"<<(int)error<<std::endl;
}
