#include "xmedia_filebrowser.h"
#include <QtCore>
#include <stdlib.h>
#include <iostream>

xmedia_filebrowser::xmedia_filebrowser(QWidget *parent, const QString &filter) : QListWidget(parent)
{
    nameFilter = filter;

    this->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
    this->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
    this->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setFrameShape(QFrame::NoFrame);
    this->setFrameShadow(QFrame::Plain);
    this->setIconSize(QSize(80, 80));
    this->setSpacing(0);

    connect(this, SIGNAL(currentItemChanged (QListWidgetItem*, QListWidgetItem*)),
            this, SLOT(itemHighlighted(QListWidgetItem*, QListWidgetItem*)));
    connect(this, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(itemSelected(QListWidgetItem*)));
    connect(this, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(itemSelected(QListWidgetItem*)));

    setDir(":home");
}


void xmedia_filebrowser::setDir(const QString &path, const QString &selected)
{
    if(path == ":home")
    {
        clear();

        new QListWidgetItem(QIcon(":/icons/file_network.png"), "Network", this, 3001);
        new QListWidgetItem(QIcon(":/icons/file_sd.png"), "SD Card", this, 3002);
        new QListWidgetItem(QIcon(":/icons/file_internal.png"), "Internal", this, 3003);

        basePath = ":home";

        this->setCurrentItem(this->item(0));
        this->clearFocus();
    }
    else
    {
        QDir dir(path, "", QDir::DirsFirst);
        if (!dir.isReadable())
            return;
        clear();

        QStringList entries = dir.entryList();
        QStringList::ConstIterator it = entries.constBegin();
        int selected_index = 0;
        for (int i = 0; it != entries.constEnd(); i++) {
            if (*it != ".")
            {
                QFileInfo fi(path + "/" + *it);

                if(selected.compare(*it) == 0)
                {
                    selected_index = i;
                }

                if(*it == "..")
                {
                    new QListWidgetItem(QIcon(":/icons/file_up.png"), "Directory Up", this, 2002);
                }
                else if(fi.isDir())
                {
                    new QListWidgetItem(QIcon(":/icons/file_folder.png"), *it, this, 2001);
                }
                else if( (fi.suffix() == "mp3" || fi.suffix() == "wav") && this->nameFilter.contains("music"))
                {
                    new QListWidgetItem(QIcon(":/icons/file_music.png"), *it, this, 1002);
                }
                else if((fi.suffix() == "mpg" || fi.suffix() == "mpeg" || fi.suffix() == "avi") && this->nameFilter.contains("video") )
                {
                    new QListWidgetItem(QIcon(":/icons/file_video.png"), *it, this, 1003);
                }
                else
                {
                    i--;
                    //DON'T Display Other Files
                }
            }
            else
            {
                i--;
            }
            ++it;
        }

        this->clearFocus();


        if(this->item(selected_index) && this->item(selected_index)->type() != 3004)
        {
            this->setCurrentItem(this->item(selected_index));
        }
        else
        {
            this->setCurrentItem(this->item(0));
        }

        basePath = dir.canonicalPath();
    }
}

void xmedia_filebrowser::itemHighlighted(QListWidgetItem *mitem, QListWidgetItem* prev)
{
    if(mitem)
    {
        if(mitem->type() == 3004)
        {
            if(prev != 0 && prev->type() != 3004)
            {
                //std::cout<<"Jumping to Prev Item with type "<<prev->type()<<" and text "<<qPrintable(prev->text())<<std::endl;
                this->scrollToItem(prev, QAbstractItemView::PositionAtCenter);
                prev->setSelected(true);
            }
        }
        else
        {
            //std::cout<<"Jumping to Item with type "<<mitem->type()<<" and text "<<qPrintable(mitem->text())<<std::endl;
            this->scrollToItem(mitem, QAbstractItemView::PositionAtCenter);
        }
    }

}

void xmedia_filebrowser::itemSelected(QListWidgetItem *mitem)
{
    if(mitem->type() == 3004)
    {
        setDir(":home"); //If something goes wrong -> go home
    }
    if(mitem->type() == 3001)
    {
        setDir(XMEDIA_FILEBROWSER_TOP_PATH + "/Network");
    }
    else if(mitem->type() == 3002)
    {
        setDir(XMEDIA_FILEBROWSER_TOP_PATH + "/SD");
    }
    else if(mitem->type() == 3003)
    {
        setDir(XMEDIA_FILEBROWSER_TOP_PATH + "/Internal");
    }
    else if(mitem->type() == 2002)
    {
        this->directoryUp();
    }
    else
    {
        QString path = basePath + "/" + mitem->text();
        if (QFileInfo(path).isDir())
        {
            if(QFileInfo(path + "../").absoluteDir() == QFileInfo(XMEDIA_FILEBROWSER_TOP_PATH).absoluteDir())
            {
                setDir(":home");
            }
            else
            {
                setDir(path);
            }
        }

        if (QFileInfo(path).isFile())
        {
            emit picked(path);
        }
    }

}

bool xmedia_filebrowser::directoryUp()
{
    if(basePath.compare(":home") == 0) //If in home directory
    {
        return false;
    }
    else
    {
       QString path = basePath + "/../";
        if (QFileInfo(path).isDir())
        {
            if(QFileInfo(path + "../").absoluteDir() == QFileInfo(XMEDIA_FILEBROWSER_TOP_PATH).absoluteDir())
            {
                setDir(":home");
            }
            else
            {
                setDir(path, QFileInfo(basePath).baseName());
            }
        }
        return true;
    }
}

int xmedia_filebrowser::scrollToItemAndCenter(int relative)
{
    int new_row = relative;

    if(currentItem())
    {
        new_row += row(currentItem());
    }

    if(new_row >= 0 && item(new_row) && item(new_row)->type() != 3004)
    {
        setCurrentItem(item(new_row));
    }
    return this->row(this->currentItem());
}

void xmedia_filebrowser::pickCurrentItem()
{
    if(currentItem() && currentItem()->type() != 3004)
    {
        itemSelected(currentItem());
    }
}

int xmedia_filebrowser::selectNextMediaFile()
{
    scrollToItemAndCenter(1);
    while(currentItem()->type() > 2000 && currentItem()->type() < 1000)
    {
        scrollToItemAndCenter(1);
    }
    pickCurrentItem();
}
int xmedia_filebrowser::selectPreviousMediaFile()
{
    scrollToItemAndCenter(-1);
    while(currentItem()->type() > 2000 && currentItem()->type() < 1000)
    {
        scrollToItemAndCenter(-1);
    }
    pickCurrentItem();
}

void xmedia_filebrowser::setFilter(const QString &filter)
{
    this->nameFilter = filter;
}

