#include "xmedia_homebrowser.h"
#include <QtCore>
#include <stdlib.h>
#include <iostream>

xmedia_homebrowser::xmedia_homebrowser(QWidget *parent) : QListWidget(parent)
{
    this->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
    this->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
    this->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setFrameShape(QFrame::NoFrame);
    this->setFrameShadow(QFrame::Plain);
    this->setIconSize(QSize(80, 80));
    this->setSpacing(0);

    connect(this, SIGNAL(currentItemChanged (QListWidgetItem*, QListWidgetItem*)),
            this, SLOT(itemHighlighted(QListWidgetItem*, QListWidgetItem*)));
    connect(this, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(itemSelected(QListWidgetItem*)));
    connect(this, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(itemSelected(QListWidgetItem*)));

    setDir(":home");
}


void xmedia_homebrowser::setDir(const QString &path, const QString &selected)
{
    if(path == ":home")
    {
        clear();

        new QListWidgetItem("Current time", this, 1002);
        new QListWidgetItem("Network Settings", this, 1003);
        new QListWidgetItem("Wifi Settings", this, 1004);
        new QListWidgetItem("LCD / VGA Mode", this, 1005);
        new QListWidgetItem("LCD Backlight", this, 1006);

        this->clearFocus();

        QTimer::singleShot(100, this, SLOT(selectFirstItem())); //Do this using a singleshot timer,
                                                                //because the scrollToItem ist not connected
    }
}

void xmedia_homebrowser::selectFirstItem()
{
    this->setCurrentItem(this->item(0));
}

void xmedia_homebrowser::itemHighlighted(QListWidgetItem *mitem, QListWidgetItem* prev)
{
    if(mitem)
    {
        if(mitem->type() == 3004)
        {
            if(prev != 0 && prev->type() != 3004)
            {
                this->scrollToItem(prev, QAbstractItemView::PositionAtCenter);
                prev->setSelected(true);
            }
        }
        else
        {
            this->scrollToItem(mitem, QAbstractItemView::PositionAtCenter);
            this->setCurrentItem(mitem);
        }
    }

}

void xmedia_homebrowser::itemSelected(QListWidgetItem *mitem)
{
    emit picked(mitem->type());
}

bool xmedia_homebrowser::directoryUp()
{
    setDir(":home");
    return true;
}

int xmedia_homebrowser::scrollToItemAndCenter(int relative)
{
    int new_row = relative;

    if(currentItem())
    {
        new_row += row(currentItem());
    }

    if(new_row >= 0 && item(new_row) && item(new_row)->type() != 3004)
    {
        setCurrentItem(item(new_row));
    }
    return this->row(this->currentItem());
}

void xmedia_homebrowser::pickCurrentItem()
{
    if(currentItem() && currentItem()->type() != 3004)
    {
        itemSelected(currentItem());
    }
}

int xmedia_homebrowser::selectNextMediaFile()
{
    scrollToItemAndCenter(1);
    while(currentItem()->type() > 2000 && currentItem()->type() < 1000)
    {
        scrollToItemAndCenter(1);
    }
    pickCurrentItem();
}
int xmedia_homebrowser::selectPreviousMediaFile()
{
    scrollToItemAndCenter(-1);
    while(currentItem()->type() > 2000 && currentItem()->type() < 1000)
    {
        scrollToItemAndCenter(-1);
    }
    pickCurrentItem();
}

