#include "xmedia_music.h"
#include "ui_xmedia_music.h"
#include "xui.h"
#include <stdlib.h>
#include <iostream>

xmedia_music::xmedia_music(QWidget *parent) :
    QDialog(parent),
    m_ui(new Ui::xmedia_music)
{
    m_ui->setupUi(this);

    volume = 60;

    player = new xMplayer();

    volume_proc = new QProcess(this);

    m_ui->filebrowser->setFilter("music, videos");

    connect(m_ui->filebrowser, SIGNAL(picked(QString)), this, SLOT(filePicked(QString)));
    connect(player, SIGNAL(finished(QString)), this, SLOT(finished(QString)));
    connect(player, SIGNAL(stateChanged(bool)), this, SLOT(stateChanged(bool)));
    connect(player, SIGNAL(fileChanged(xMPlayerFileInfo)), this, SLOT(fileChanged(xMPlayerFileInfo)));
    connect(player, SIGNAL(progressChanged(xMPlayerProgress)), this, SLOT(progressChanged(xMPlayerProgress)));
}

xmedia_music::~xmedia_music()
{
    delete volume_proc;
    delete player;
    delete m_ui;
}

void xmedia_music::startPlayback(QString file)
{
    if(file != "")
    {
        this->player->play(file);

        if(QFileInfo(QFileInfo(file).absolutePath() + "/Folder.jpg").isFile())
        {
            this->m_ui->img_folder->setPixmap(QPixmap(QFileInfo(file).absolutePath() + "/Folder.jpg"));
        }
        else
        {
             this->m_ui->img_folder->setPixmap(QPixmap(":/icons/file_music.png"));
        }
    }
    else
    {
        this->player->setPlay(1);
    }
}

void xmedia_music::volumeSet(int relative_volume)
{
    volume += relative_volume;
    if(volume < 0)
    {
        volume = 0;
    }
    if(volume > 100)
    {
        volume = 100;
    }

    char data[30];
    sprintf(data, "amixer sset PCM %i%% unmute", volume);
    volume_proc->start(data);
    volume_proc->waitForFinished(30000);
    m_ui->progressBar_2->setValue(volume);
}

void xmedia_music::startStopPlayback()
{
    this->player->setPlay();
}

void xmedia_music::stopPlayback()
{
    this->player->setPlay(0);
}

void xmedia_music::commandUserInterfaceReceived(unsigned char command, unsigned char length, unsigned char *data )
{
    if(m_ui->stackedWidget->currentIndex() == 0) //If in Browser
    {
        this->userInterfaceBrowser(command);
    }
    else if(m_ui->stackedWidget->currentIndex() == 1) //If in Player
    {
        this->userInterfaceMusicPlayer(command);
    }
    else if(m_ui->stackedWidget->currentIndex() == 2) //If in Video Player
    {
        this->userInterfaceVideoPlayer(command);
    }
    else
    {
        this->userInterfaceVolume(command);
    }
}

void xmedia_music::userInterfaceBrowser(unsigned char command)
{
    if(command == XUI_RIGHT || command == XUI_4WAY_DOWN)
    {
        m_ui->filebrowser->scrollToItemAndCenter(-1);
    }
    else if(command == XUI_LEFT || command == XUI_4WAY_UP)
    {
        m_ui->filebrowser->scrollToItemAndCenter(1);
    }
    else if(command == XUI_OK || command == XUI_4WAY_RIGHT)
    {
        m_ui->filebrowser->pickCurrentItem();
    }
    else if(command == XUI_BACK || command == XUI_4WAY_LEFT)
    {
       if(!m_ui->filebrowser->directoryUp())
       {
           accept();
       }
    }
    else if(command == XUI_GP2)
    {
        this->gotoPlayer();
    }
    else if(command == XUI_GP1)
    {
        this->gotoVolume();
    }
}
void xmedia_music::userInterfaceMusicPlayer(unsigned char command)
{
    if(command == XUI_RIGHT || command == XUI_4WAY_DOWN)
    {
        volumeSet(-5); //Just lower the volume
        //m_ui->filebrowser->selectPreviousMediaFile();
    }
    else if(command == XUI_LEFT || command == XUI_4WAY_UP)
    {
        //m_ui->filebrowser->selectNextMediaFile();
        volumeSet(+5);
    }
    else if(command == XUI_OK || command == XUI_4WAY_RIGHT)
    {
        startStopPlayback();
    }
    else if(command == XUI_BACK || command == XUI_4WAY_LEFT)
    {
        this->gotoBrowser();
    }
    else if(command == XUI_GP2)
    {
        this->gotoBrowser();
    }
    else if(command == XUI_GP1)
    {
        this->gotoVolume();
    }
}
void xmedia_music::userInterfaceVideoPlayer(unsigned char command)
{
    if(command == XUI_RIGHT || command == XUI_4WAY_DOWN)
    {
        volumeSet(-5); //Just lower the volume
        //m_ui->filebrowser->selectPreviousMediaFile();
    }
    else if(command == XUI_LEFT || command == XUI_4WAY_UP)
    {
        //m_ui->filebrowser->selectNextMediaFile();
        volumeSet(+5);
    }
    else if(command == XUI_OK || command == XUI_4WAY_RIGHT)
    {
        startStopPlayback();
    }
    else if(command == XUI_BACK || command == XUI_4WAY_LEFT)
    {
        stopPlayback();
        this->gotoBrowser();
    }
    else if(command == XUI_GP2)
    {
        stopPlayback();
        this->gotoBrowser();
    }
    else if(command == XUI_GP1)
    {
        stopPlayback();
        this->gotoVolume();
    }
}
void xmedia_music::userInterfaceVolume(unsigned char command)
{
    if(command == XUI_RIGHT || command == XUI_4WAY_DOWN)
    {
        volumeSet(-5);
    }
    else if(command == XUI_LEFT || command == XUI_4WAY_UP)
    {
        volumeSet(+5);
    }
    else if(command == XUI_OK || command == XUI_4WAY_RIGHT)
    {

    }
    else if(command == XUI_BACK || command == XUI_4WAY_LEFT)
    {
        this->gotoPlayer();
    }
    else if(command == XUI_GP2)
    {
        this->gotoBrowser();
    }
    else if(command == XUI_GP1)
    {
        this->gotoPlayer();
    }
}

void xmedia_music::gotoBrowser()
{
    m_ui->stackedWidget->setCurrentIndex(0);
}
void xmedia_music::gotoPlayer()
{
    if(this->player->getMediaType() == XMPLAYER_AUDIO)
    {
        m_ui->stackedWidget->setCurrentIndex(1);
    }
    else
    {
        m_ui->stackedWidget->setCurrentIndex(2);
    }
}
void xmedia_music::gotoVolume()
{
    m_ui->stackedWidget->setCurrentIndex(3);
}


void xmedia_music::changeEvent(QEvent *e)
{
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void xmedia_music::filePicked(const QString &file)
{
    startPlayback(file);
    gotoPlayer();
}

void xmedia_music::stateChanged(bool playing)
{
    if(playing)
    {

    }
    else
    {

    }
}

void xmedia_music::progressChanged(xMPlayerProgress progress)
{
    QTime base(0,0);
    QTime timeElapsed = base.addSecs(progress.secondsPlayed);
    QTime timeRemaining = base.addSecs(progress.secondsTotal - progress.secondsPlayed);
    timeRemaining.addSecs(progress.secondsTotal - progress.secondsPlayed);
    m_ui->lbl_time_elapsed->setText(timeElapsed.toString("mm:ss"));
    m_ui->lbl_time_remaining->setText(timeRemaining.toString("mm:ss"));

    m_ui->progressBar->setValue((int)(progress.percentPlayed));
}

void xmedia_music::fileChanged(xMPlayerFileInfo info)
{
    m_ui->lbl_filename->setText(QFileInfo(info.filename).fileName());
    m_ui->lbl_title->setText(info.title);
    m_ui->lbl_interpret->setText(info.artist);
    m_ui->lbl_album->setText(info.album);
    m_ui->lbl_year->setText(info.year);
}

void xmedia_music::finished(QString filename)
{
    m_ui->filebrowser->selectNextMediaFile();
}
