#include "xmedia_stream.h"
#include "ui_xmedia_stream.h"
#include "xui.h"
#include <stdlib.h>
#include <iostream>

xmedia_stream::xmedia_stream(QWidget *parent) :
    QDialog(parent),
    m_ui(new Ui::xmedia_stream)
{
    m_ui->setupUi(this);

    volume = 60;

    player = new xMplayer();

    volume_proc = new QProcess(this);

    connect(m_ui->streambrowser, SIGNAL(picked(QString)), this, SLOT(filePicked(QString)));
    connect(player, SIGNAL(finished(QString)), this, SLOT(finished(QString)));
    connect(player, SIGNAL(stateChanged(bool)), this, SLOT(stateChanged(bool)));
    connect(player, SIGNAL(fileChanged(xMPlayerFileInfo)), this, SLOT(fileChanged(xMPlayerFileInfo)));
    connect(player, SIGNAL(progressChanged(xMPlayerProgress)), this, SLOT(progressChanged(xMPlayerProgress)));
}

xmedia_stream::~xmedia_stream()
{
    delete volume_proc;
    delete player;
    delete m_ui;
}

void xmedia_stream::startPlayback(QString file)
{
    if(file != "")
    {
        this->player->play(file);
        setStatus(XMEDIA_STREAM_BUFFERING);

        if(QFileInfo(QFileInfo(file).absolutePath() + "/Folder.jpg").isFile())
        {
            this->m_ui->img_folder->setPixmap(QPixmap(QFileInfo(file).absolutePath() + "/Folder.jpg"));
        }
        else
        {
             this->m_ui->img_folder->setPixmap(QPixmap(":/icons/file_music.png"));
        }
    }
    else
    {
        this->player->setPlay(1);
    }
}

void xmedia_stream::volumeSet(int relative_volume)
{
    volume += relative_volume;
    if(volume < 0)
    {
        volume = 0;
    }
    if(volume > 100)
    {
        volume = 100;
    }

    char data[30];
    sprintf(data, "amixer sset PCM %i%% unmute -q", volume);
    volume_proc->startDetached(data);
    //volume_proc->waitForFinished(30000);
    m_ui->progressBar_2->setValue(volume);
}

void xmedia_stream::startStopPlayback()
{
    this->player->setPlay();
}

void xmedia_stream::stopPlayback()
{
    this->player->setPlay(0);
}

void xmedia_stream::commandUserInterfaceReceived(unsigned char command, unsigned char length, unsigned char *data )
{
    if(m_ui->stackedWidget->currentIndex() == 0) //If in Browser
    {
        this->userInterfaceBrowser(command);
    }
    else if(m_ui->stackedWidget->currentIndex() == 1) //If in Player
    {
        this->userInterfaceMusicPlayer(command);
    }
    else if(m_ui->stackedWidget->currentIndex() == 2) //If in Video Player
    {
        this->userInterfaceVideoPlayer(command);
    }
}

void xmedia_stream::userInterfaceBrowser(unsigned char command)
{
    if(command == XUI_RIGHT || command == XUI_4WAY_DOWN)
    {
        m_ui->streambrowser->scrollToItemAndCenter(-1);
    }
    else if(command == XUI_LEFT || command == XUI_4WAY_UP)
    {
        m_ui->streambrowser->scrollToItemAndCenter(1);
    }
    else if(command == XUI_OK || command == XUI_4WAY_RIGHT)
    {
        m_ui->streambrowser->pickCurrentItem();
    }
    else if(command == XUI_BACK || command == XUI_4WAY_LEFT)
    {
        accept();
    }
    else if(command == XUI_GP2)
    {
        this->gotoPlayer();
    }
    else if(command == XUI_GP1)
    {

    }
}
void xmedia_stream::userInterfaceMusicPlayer(unsigned char command)
{
    if(command == XUI_RIGHT || command == XUI_4WAY_DOWN)
    {
        volumeSet(-5); //Just lower the volume
        //m_ui->streambrowser->selectPreviousMediaFile();
    }
    else if(command == XUI_LEFT || command == XUI_4WAY_UP)
    {
        //m_ui->streambrowser->selectNextMediaFile();
        volumeSet(+5);
    }
    else if(command == XUI_OK || command == XUI_4WAY_RIGHT)
    {
        startStopPlayback();
    }
    else if(command == XUI_BACK || command == XUI_4WAY_LEFT)
    {
        this->gotoBrowser();
    }
    else if(command == XUI_GP2)
    {
        this->gotoBrowser();
    }
    else if(command == XUI_GP1)
    {

    }
}
void xmedia_stream::userInterfaceVideoPlayer(unsigned char command)
{
    if(command == XUI_RIGHT || command == XUI_4WAY_DOWN)
    {
        volumeSet(-5); //Just lower the volume
        //m_ui->streambrowser->selectPreviousMediaFile();
    }
    else if(command == XUI_LEFT || command == XUI_4WAY_UP)
    {
        //m_ui->streambrowser->selectNextMediaFile();
        volumeSet(+5);
    }
    else if(command == XUI_OK || command == XUI_4WAY_RIGHT)
    {
        startStopPlayback();
    }
    else if(command == XUI_BACK || command == XUI_4WAY_LEFT)
    {
        stopPlayback();
        this->gotoBrowser();
    }
    else if(command == XUI_GP2)
    {
        stopPlayback();
        this->gotoBrowser();
    }
    else if(command == XUI_GP1)
    {

    }
}

void xmedia_stream::gotoBrowser()
{
    m_ui->stackedWidget->setCurrentIndex(0);
}
void xmedia_stream::gotoPlayer()
{
    if(this->player->getMediaType() != XMPLAYER_VIDEO)
    {
        m_ui->stackedWidget->setCurrentIndex(1);
    }
    else
    {
        m_ui->stackedWidget->setCurrentIndex(2);
    }
}


void xmedia_stream::changeEvent(QEvent *e)
{
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void xmedia_stream::filePicked(const QString &file)
{
    startPlayback(file);
    gotoPlayer();
}

void xmedia_stream::stateChanged(bool playing)
{
    if(playing)
    {
        setStatus(XMEDIA_STREAM_PLAYING);
    }
    else
    {
        setStatus(XMEDIA_STREAM_PAUSED);
    }
}

void xmedia_stream::setStatus(XMEDIA_STREAM_STATUS status)
{
    QString message = "";

    if(status == XMEDIA_STREAM_BUFFERING)
    {
        message = "Buffering...";
    }
    else if(status == XMEDIA_STREAM_PLAYING)
    {
        message = "Playing...";
    }
    else if(status == XMEDIA_STREAM_PAUSED)
    {
        message = "Paused";
    }
    else if(status == XMEDIA_STREAM_ENDED)
    {
        message = "Ended";
    }

    this->m_ui->lbl_status->setText(message);
}

void xmedia_stream::progressChanged(xMPlayerProgress progress)
{
    QTime base(0,0);
    QTime timeElapsed = base.addSecs(progress.secondsPlayed);
    m_ui->lbl_time_elapsed->setText(timeElapsed.toString("mm:ss"));
}

void xmedia_stream::fileChanged(xMPlayerFileInfo info)
{
    setStatus(XMEDIA_STREAM_PLAYING);

    xmedia_config *config = new xmedia_config();

    for(int i = 0; i < config->stations.size(); i++)
    {
        if(info.filename == config->stations[i].address)
        {
            m_ui->lbl_title->setText(config->stations[i].name);
        }
    }

    delete config;
}

void xmedia_stream::finished(QString filename)
{
    setStatus(XMEDIA_STREAM_ENDED);
}
