#include "xmedia_streambrowser.h"
#include <QtCore>
#include <stdlib.h>
#include <iostream>

xmedia_streambrowser::xmedia_streambrowser(QWidget *parent) : QListWidget(parent)
{
    this->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
    this->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
    this->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setFrameShape(QFrame::NoFrame);
    this->setFrameShadow(QFrame::Plain);
    this->setIconSize(QSize(80, 80));
    this->setSpacing(0);

    connect(this, SIGNAL(currentItemChanged (QListWidgetItem*, QListWidgetItem*)),
            this, SLOT(itemHighlighted(QListWidgetItem*, QListWidgetItem*)));
    connect(this, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(itemSelected(QListWidgetItem*)));
    connect(this, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(itemSelected(QListWidgetItem*)));

    setDir(":home");
}


void xmedia_streambrowser::setDir(const QString &path, const QString &selected)
{
    if(path == ":home")
    {
        clear();

        xmedia_config *config = new xmedia_config();

        for(int i = 0; i < config->stations.size(); i++)
        {
            new QListWidgetItem(QIcon(":/icons/file_music.png"), config->stations[i].name, this, 1002);
        }

        delete config;

        this->clearFocus();

        QTimer::singleShot(100, this, SLOT(selectFirstItem())); //Do this using a singleshot timer,
                                                                //because the scrollToItem ist not connected
    }
}

void xmedia_streambrowser::selectFirstItem()
{
    this->setCurrentItem(this->item(0));
}

void xmedia_streambrowser::itemHighlighted(QListWidgetItem *mitem, QListWidgetItem* prev)
{
    if(mitem)
    {
        if(mitem->type() == 3004)
        {
            if(prev != 0 && prev->type() != 3004)
            {
                //std::cout<<"Jumping to Prev Item with type "<<prev->type()<<" and text "<<qPrintable(prev->text())<<std::endl;
                this->scrollToItem(prev, QAbstractItemView::PositionAtCenter);
                prev->setSelected(true);
            }
        }
        else
        {
            //std::cout<<"Jumping to Item with type "<<mitem->type()<<" and text "<<qPrintable(mitem->text())<<std::endl;
            this->scrollToItem(mitem, QAbstractItemView::PositionAtCenter);
            this->setCurrentItem(mitem);
        }
    }

}

void xmedia_streambrowser::itemSelected(QListWidgetItem *mitem)
{

    xmedia_config *config = new xmedia_config();

    for(int i = 0; i < config->stations.size(); i++)
    {
        if(config->stations[i].name == mitem->text())
        {
            emit picked(config->stations[i].address);
            break;
        }
    }

    delete config;

}

bool xmedia_streambrowser::directoryUp()
{
    setDir(":home");
    return true;
}

int xmedia_streambrowser::scrollToItemAndCenter(int relative)
{
    int new_row = relative;

    if(currentItem())
    {
        new_row += row(currentItem());
    }

    if(new_row >= 0 && item(new_row) && item(new_row)->type() != 3004)
    {
        setCurrentItem(item(new_row));
    }
    return this->row(this->currentItem());
}

void xmedia_streambrowser::pickCurrentItem()
{
    if(currentItem() && currentItem()->type() != 3004)
    {
        itemSelected(currentItem());
    }
}

int xmedia_streambrowser::selectNextMediaFile()
{
    scrollToItemAndCenter(1);
    while(currentItem()->type() > 2000 && currentItem()->type() < 1000)
    {
        scrollToItemAndCenter(1);
    }
    pickCurrentItem();
}
int xmedia_streambrowser::selectPreviousMediaFile()
{
    scrollToItemAndCenter(-1);
    while(currentItem()->type() > 2000 && currentItem()->type() < 1000)
    {
        scrollToItemAndCenter(-1);
    }
    pickCurrentItem();
}

