#include "xmplayer.h"
#include <stdlib.h>
#include <iostream>

xMplayer::xMplayer()
{
    mplayer = new QProcess();
    connect( mplayer, SIGNAL( finished(int, QProcess::ExitStatus) ), this, SLOT( exitedMPlayer(int, QProcess::ExitStatus) ) );
    connect( mplayer, SIGNAL( readyReadStandardError() ), this, SLOT( readyReadStderrMPlayer() ) );
    connect( mplayer, SIGNAL( readyReadStandardOutput() ), this, SLOT( readStdoutMPlayer() ) );
}

xMplayer::~xMplayer()
{
    mplayer->write("q");
    mplayer->waitForFinished(1000);
    mplayer->kill(); //Kill prevKillingious instances - hard and cruel
    mplayer->waitForFinished(30000);

    delete mplayer;
}

void xMplayer::play(QString filename)
{
    this->info.filename = filename;

    this->info.artist = "";
    this->info.album = "";
    this->info.title = "";
    this->info.year = "";
    this->info.comment = "";
    this->info.track = "";
    this->info.genre = "";

    this->paused = true;

    if(mplayer->pid() > 0)
    {
        mplayer->write("q");
        mplayer->waitForFinished(1000);
        mplayer->kill(); //Kill prevKillingious instances - hard and cruel
        mplayer->waitForFinished(30000);
    }


    if(QFileInfo(filename).suffix() == "wav" || QFileInfo(filename).suffix() == "mp3" || QFileInfo(filename).suffix() == "ogg")
    {
        this->type = XMPLAYER_AUDIO;
    }
    else if(QFileInfo(filename).suffix() == "mpeg" || QFileInfo(filename).suffix() == "mpg" || QFileInfo(filename).suffix() == "avi")
    {
        this->type = XMPLAYER_VIDEO;
    }
    else
    {
        this->type = XMPLAYER_UNKNOWN;
    }


    QString audiocodec = "";   

    if(QFileInfo(filename).suffix() != "wav" && QFileInfo(filename).suffix() != "ogg")
    {
        audiocodec = "-ac mad ";
    }

    mplayer->start("mplayer " + audiocodec + " -flip -framedrop -fs -af channels=2 \"" + this->info.filename + "\""); //-vop mirror
    //mplayer -ac mad -framedrop -fs -af channels=2
    this->paused = false;
}

xMPlayerMediaType xMplayer::getMediaType()
{
    return this->type;
}

void xMplayer::setPlay(int play)
{
    if(play == -1)
    {
        play = this->paused;
    }

    if(this->paused && play)
    {
        mplayer->write("s");
        mplayer->waitForReadyRead(200);
        this->paused = false;
        this->stateChanged(!this->paused);
    }
    else if(!this->paused && !play)
    {
        mplayer->write("p");
        mplayer->waitForReadyRead(250);
        this->paused = true;
        this->stateChanged(!this->paused);
    }

}

xMPlayerProgress xMplayer::getProgress()
{
    return this->progress;
}

void xMplayer::setProgress(int percent)
{

}

void xMplayer::exitedMPlayer(int exitCode, QProcess::ExitStatus exitStatus)
{
    if(!this->paused)
    {
        this->finished(this->info.filename);
    }
}

void xMplayer::wroteToStdinMPlayer()
{

}

void xMplayer::readyReadStderrMPlayer()
{

}
void xMplayer::readStdoutMPlayer()
{
    QString out;
    int tmp;

    if ( mplayer->canReadLine() )
    {
        out = mplayer->readLine();

        if(out.indexOf("Title:") != -1)
        {
            this->info.title = out.mid(out.indexOf(":")+2);
        }
        if(out.indexOf("Artist:") != -1)
        {
            this->info.artist = out.mid(out.indexOf(":")+2);
        }
        if(out.indexOf("Album:") != -1)
        {
            this->info.album = out.mid(out.indexOf(":")+2);
        }
        if(out.indexOf("Year:") != -1)
        {
            this->info.year = out.mid(out.indexOf(":")+2);
        }
        if(out.indexOf("Comment:") != -1)
        {
            this->info.comment = out.mid(out.indexOf(":")+2);
        }
        if(out.indexOf("Track:") != -1)
        {
            this->info.track = out.mid(out.indexOf(":")+2);
        }
        if(out.indexOf("Genre:") != -1)
        {
            this->info.genre = out.mid(out.indexOf(":")+2);
        }
        if(out.indexOf("Starting playback") != -1)
        {
            this->fileChanged(this->info);
        }
    }
    else
    {
        out =  mplayer->readAll(); //One complete line is transmitted at once - so we dont have to buffer anything =)

        if(out.indexOf("A:") != -1)
        {
            this->progress.secondsPlayed = out.mid(out.indexOf("A:")+3, 6).toDouble();
            this->progress.secondsTotal = out.mid(out.indexOf("of")+3, 6).toDouble();

            this->progress.percentPlayed = this->progress.secondsPlayed / this->progress.secondsTotal * 100.0;
            this->progressChanged(this->progress);
        }
    }
}
