#ifndef XMPLAYER_H
#define XMPLAYER_H

#include <QObject>
#include <QProcess>
#include <QDir>

typedef struct
{
    QString filename;
    QString artist;
    QString album;
    QString title;
    QString year;
    QString comment;
    QString track;
    QString genre;
} xMPlayerFileInfo;

typedef struct
{
    double secondsPlayed;
    double secondsTotal;
    double percentPlayed;
} xMPlayerProgress;

typedef enum
{
    XMPLAYER_AUDIO,
    XMPLAYER_VIDEO,
    XMPLAYER_UNKNOWN
} xMPlayerMediaType;


class xMplayer : public QObject
{
    Q_OBJECT

    public:
        xMplayer();
        ~xMplayer();
        void play(QString filename);
        void setPlay(int play = -1);
        xMPlayerProgress getProgress();
        void setProgress(int percent);
        xMPlayerMediaType getMediaType();

    signals:
        void fileChanged(xMPlayerFileInfo);
        void finished(QString);
        void progressChanged(xMPlayerProgress);
        void stateChanged(bool);

    private slots:
        void exitedMPlayer(int exitCode, QProcess::ExitStatus exitStatus);
        void wroteToStdinMPlayer();
        void readyReadStderrMPlayer();
        void readStdoutMPlayer();

     private:
        QProcess *mplayer;
        xMPlayerFileInfo info;
        xMPlayerProgress progress;
        xMPlayerMediaType type;
        int paused;
        QString ibuffer;
        



};

#endif // XMPLAYER_H
