#include "xsie.h"
#include <iostream>
#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */


xsie::xsie()
{
    serialPortInit();
    Enabled = true;
    State = XSIE_IDLE;
}



void xsie::run()
{
    serialPortRead();
}


unsigned char xsie::crcCreate(xsieRawPacket data, unsigned char length)
{
    //TODO ADD CRC generation
    return 0;
}

unsigned char xsie::crcCheck(xsiePacket data)
{
    //TODO ADD CRC Check
    return 1;
}

unsigned char xsie::packData()
{
    xsiePacket packet;
    int i = 0;

    packet.pid 	= ReceiveTemp[0] << 8 | ReceiveTemp[1];
    packet.command 	= ReceiveTemp[2];
    packet.length	= ReceiveTemp[3];

    for(i = 0; i < packet.length && i < XSIE_MAX_PACKET_SIZE; i++)
    {
            packet.data[i] = ReceiveTemp[i + 4];
    }

    packet.crc		= ReceiveTemp[i+4];


    if(!crcCheck(packet))
    {
            dataError(XSIE_CRC_CHECK_FAILED);
            return 0;
    }
    else
    {
            packetReceived(packet);
            return 1;
    }

}


unsigned char xsie::packRaw(xsieRawPacket buffer, unsigned char pid, unsigned char command, unsigned char length, unsigned char *data)
{
    int i = 0;

    buffer[0] = pid >> 8; //MSB
    buffer[1] = pid & 0x00FF; // LSB
    buffer[2] = command;


    for(i = 0; i < length && i < XSIE_MAX_PACKET_SIZE; i++)
    {
            buffer[i+4] = data[i];
    }

    buffer[3] = i;

    buffer[4+i] = crcCreate(buffer, i+3);

    return i+5;
}


void xsie::submitRaw(xsieRawPacket raw, unsigned char length)
{
    int i = 0;

    /* Transmission MUST NOT be disturbed */
    for(i = 0; i < length; i++)
    {
            if(raw[i] == XSIE_END)
            {
                    serialPortWrite(XSIE_ESC);
                    serialPortWrite(XSIE_ESC_END);
            }
            else if(raw[i] == XSIE_ESC)
            {
                    serialPortWrite(XSIE_ESC);
                    serialPortWrite(XSIE_ESC_ESC);
            }
            else
            {
                    serialPortWrite(raw[i]);
            }
    }
    serialPortWrite(XSIE_END);
}

void xsie::submitPacket(xsiePacket data)
{
    xsieRawPacket buffer;
    unsigned char plength = 0;
    plength = xsie::packRaw(buffer, data.pid, data.command, data.length, data.data);
    xsie::submitRaw(buffer, plength);

}

void xsie::submit(unsigned char command, unsigned char length, unsigned char *data)
{
    xsieRawPacket buffer;
    unsigned char plength = 0;
    plength = xsie::packRaw(buffer, PacketId++, command, length, data);
    xsie::submitRaw(buffer, plength);
}

void xsie::respond(unsigned char pid, unsigned char command, unsigned char length, unsigned char *data)
{
    xsieRawPacket buffer;
    unsigned char plength = 0;
    plength = xsie::packRaw(buffer, pid | (1<<15), command, length, data);
    xsie::submitRaw(buffer, plength);
}

void xsie::receive(unsigned char data)
{
     switch (State)
     {
        case XSIE_IDLE:
        {
             if(data == XSIE_END) //End of invalid transmission detected
             {
                                dataError(XSIE_ILLEGAL_FRAMESTART);

                                int i = 0;
                                for(i = 0; i < XSIE_MAX_PACKET_LENGTH; i++)
                                {
                                        ReceiveTemp[i] = 0;
                                }
                                Framestart = 0;

                    State = XSIE_RECEIVING;
             }

             break;
        }
        case XSIE_RECEIVING:
        {
             if(data == XSIE_ESC)
             {
                     State = XSIE_ESCAPING;
             }
             else if(data == XSIE_END)
             {
                                packData();

                                int i = 0;
                                for(i = 0; i < XSIE_MAX_PACKET_LENGTH; i++)
                                {
                                        ReceiveTemp[i] = 0;
                                }
                                Framestart = 0;

                                State = XSIE_RECEIVING;
                 }
             else
             {
                     ReceiveTemp[Framestart++] = data;

                     State = XSIE_RECEIVING;
             }

             break;
        }
        case XSIE_ESCAPING:
        {
                if(data == XSIE_ESC_END)
                {
                        ReceiveTemp[Framestart++] = XSIE_END;
                        State = XSIE_RECEIVING;
                }
                else if(data == XSIE_ESC_ESC)
                {
                        ReceiveTemp[Framestart++] = XSIE_END;
                        State = XSIE_RECEIVING;
                }
                else
                {
                        dataError(XSIE_ILLEGAL_ESCAPE_SEQUENCE);

                        State = XSIE_IDLE;
                }
        }
     }
}


void xsie::serialPortInit()
{
      struct termios options;

      SerialPort = open(XSIE_USE_DEV, O_RDWR | O_NOCTTY | O_NDELAY);

      if(SerialPort == -1)
      {
          std::cerr<<"Unable to open serial port "<<XSIE_USE_DEV<<"!"<<std::endl;
          return;
      }


     tcgetattr(SerialPort, &options);
     cfsetispeed(&options, XSIE_USE_BAUD);
     cfsetospeed(&options, XSIE_USE_BAUD);

     cfmakeraw(&options);/// !!!!! GO TO BINARY TRANSFER!!!

     tcsetattr(SerialPort, TCSANOW, &options);

     fcntl(SerialPort, F_SETFL, 0);
}

void xsie::serialPortWrite(unsigned char data)
{
    unsigned char temp_data[2];

    temp_data[0] = data;

    write(SerialPort, temp_data, 1);
}

void xsie::serialPortRead()
{
    unsigned char data_tmp[2];
    unsigned int ncount = 0;


    while((ncount = read(SerialPort, data_tmp, 1)) > 0) //Just read one single byte from the buffer
    {
        receive(data_tmp[0]);
    }
}
