#ifndef XSIE_H
#define XSIE_H

#include <QThread>

#define XSIE_USE_DEV "/dev/ttyS1"
#define XSIE_USE_BAUD B38400 /*B2400*/
#define XSIE_MAX_PACKET_SIZE 128
#define XSIE_CONTROL_BYTES 5
#define XSIE_MAX_PACKET_LENGTH (XSIE_MAX_PACKET_SIZE + XSIE_CONTROL_BYTES)
#define XSIE_END             0300    /* indicates end of packet */
#define XSIE_ESC             0333    /* indicates byte stuffing */
#define XSIE_ESC_END         0334    /* ESC ESC_END means END data byte */
#define XSIE_ESC_ESC         0335    /* ESC ESC_ESC means ESC data byte */

typedef struct
{
        unsigned int pid;
        unsigned char command;
        unsigned char length;
        unsigned char data[XSIE_MAX_PACKET_SIZE];
        unsigned char crc;

} xsiePacket;

typedef unsigned char xsieRawPacket[XSIE_MAX_PACKET_LENGTH];

typedef enum
{
        XSIE_IDLE,
        XSIE_RECEIVING,
        XSIE_ESCAPING

} xsieState;

typedef enum
{
        XSIE_ILLEGAL_FRAMESTART,
        XSIE_ILLEGAL_ESCAPE_SEQUENCE,
        XSIE_CRC_CHECK_FAILED
} xsieError;

typedef enum
{
        XSIE_NORMAL,
        XSIE_ANSWER
} xsiePacketType;


class xsie : public QThread
{
    Q_OBJECT

public:
    xsie();
    void submit(unsigned char command, unsigned char length, unsigned char *data);
    void respond(unsigned char pid, unsigned char command, unsigned char length, unsigned char *data);
    void submitPacket(xsiePacket data);
    void receive(unsigned char data);

protected:
    void run();

signals:
    void packetReceived(xsiePacket);
    void dataError(xsieError);

private:
    void serialPortInit();
    void serialPortRead();
    void serialPortWrite(unsigned char data);
    unsigned char packData();
    unsigned char packRaw(xsieRawPacket buffer, unsigned char pid, unsigned char command, unsigned char length, unsigned char *data);
    void submitRaw(xsieRawPacket raw, unsigned char length);

    unsigned char crcCreate(xsieRawPacket data, unsigned char length);
    unsigned char crcCheck(xsiePacket data);


    int SerialPort;
    volatile bool Enabled;
    xsieState State;
    unsigned char Framestart;
    unsigned char ReceiveTemp[XSIE_MAX_PACKET_LENGTH];
    int PacketId;



};

#endif // XSIE_H
